// Krogla.cpp: implementation of the CKrogla class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ZBuffer.h"
#include "Krogla.h"
#include <math.h>

#define COS(x) (cos(3.141592*x/180))
#define SIN(x) (sin(3.141592*x/180))

#define SWAP(a,b) temp=(a);(a)=(b);(b)=temp;
#define M 7
#define NSTACK 50


#define PI 3.141592

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CKrogla::CKrogla()
{

}

CKrogla::CKrogla(double dRadij, CVector cvVector, double dFi, double dTheta, int iSteviloPoldnevnikov, int iSteviloVzporednikov, double dOddaljenostOdZaslona, UINT uiDelay)
{
	m_dRadij = dRadij;
	m_cvPremik = cvVector;
	m_ddFi = dFi;
	m_ddTheta = dTheta;
	m_iSteviloPoldnevnikov = iSteviloPoldnevnikov;
	m_iSteviloVzporednikov = iSteviloVzporednikov;
	m_ddFi = 2*PI/((double)iSteviloPoldnevnikov);
	m_ddTheta = PI/((double)iSteviloVzporednikov)/2.0;
	m_dOddaljenostOdZaslona = dOddaljenostOdZaslona;
	m_uiDelay = uiDelay;
}

CKrogla::~CKrogla()
{

}

void CKrogla::SetRadij(double dRadij)
{
	m_dRadij = dRadij;
}

void CKrogla::SetPremik(CVector cvVector)
{
	m_cvPremik = cvVector;
}

void CKrogla::SetdFi(double dFi)
{
	m_ddFi = dFi;
}

void CKrogla::SetdTheta(double dTheta)
{
	m_ddTheta = dTheta;
}

double CKrogla::GetRadij()
{
	return m_dRadij;
}

CVector CKrogla::GetPremik()
{
	return m_cvPremik;
}

void CKrogla::SetPoldnevniskaDelitev(int iSteviloPoldnevnikov)
{
	m_iSteviloPoldnevnikov = iSteviloPoldnevnikov;
}

void CKrogla::SetVzporedniskaDelitev(int iSteviloVzporednikov)
{
	m_iSteviloVzporednikov = iSteviloVzporednikov;
}

void CKrogla::SetOddaljenostOdZaslona(double dOddaljenostOdZaslona)
{
	m_dOddaljenostOdZaslona = dOddaljenostOdZaslona;
}

int CKrogla::GetPoldnevniskaDelitev()
{
	return m_iSteviloPoldnevnikov;
}

int CKrogla::GetVzporedniskaDelitev()
{
	return m_iSteviloVzporednikov;
}

double CKrogla::GetdFi()
{
	return m_ddFi;
}

double CKrogla::GetdTheta()
{
	return m_ddTheta;
}

double CKrogla::GetOddaljenostOdZaslona()
{
	return m_dOddaljenostOdZaslona;
}

//  *************************  RAZDELITEV POVRINE KROGLE
void CKrogla::RazdeliSe()
{
	double Fi,Theta;

	CVector cvBaseVector, cvRightVector, cvRightTopVector, cvTopVector;
	CPloskev cpPloskev;
	double x,y,z;
	int j,i;

	// spodnja vrsta trikotniki
	Fi = 2*PI;
	Theta = PI - m_ddTheta;
	for (i = 1; i < m_iSteviloPoldnevnikov; i++)
	{
		x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi);
		y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi);
		z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
		cvBaseVector.SetXYZ(x,y,z);
		cvBaseVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi + m_ddFi);
		y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi + m_ddFi);
		z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
		cvRightVector.SetXYZ(x,y,z);
		cvRightVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX();
		y = m_cvPremik.GetY();
		z = m_cvPremik.GetZ() - m_dRadij;
		cvRightTopVector.SetXYZ(x,y,z);
		cvRightTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX();
		y = m_cvPremik.GetY();
		z = m_cvPremik.GetZ() - m_dRadij;
		cvTopVector.SetXYZ(x,y,z);
		cvTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		cpPloskev.SetBaseVector(cvBaseVector);
		cpPloskev.SetRightVector(cvRightVector);
		cpPloskev.SetTopRightVector(cvRightTopVector);
		cpPloskev.SetTopVector(cvTopVector);

		m_arPloskev.Add(cpPloskev);

		Fi -= m_ddFi;
	}

	//	delitev srednjega dela
	Fi = 0;
	Theta = PI - m_ddTheta;
	for ( i = 1; i < 2*m_iSteviloVzporednikov; i++)
	{
		Fi = 0;
		for ( j = 1; j <= m_iSteviloPoldnevnikov; j++)
		{
			x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi);
			y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi);
			z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
			cvBaseVector.SetXYZ(x,y,z);
			cvBaseVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

			x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi + m_ddFi);
			y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi + m_ddFi);
			z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
			cvRightVector.SetXYZ(x,y,z);
			cvRightVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

			x = m_cvPremik.GetX() + m_dRadij*sin(Theta - m_ddTheta)*cos(Fi + m_ddFi);
			y = m_cvPremik.GetY() + m_dRadij*sin(Theta - m_ddTheta)*sin(Fi + m_ddFi);
			z = m_cvPremik.GetZ() + m_dRadij*cos(Theta - m_ddTheta);
			cvRightTopVector.SetXYZ(x,y,z);
			cvRightTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

			x = m_cvPremik.GetX() + m_dRadij*sin(Theta - m_ddTheta)*cos(Fi);
			y = m_cvPremik.GetY() + m_dRadij*sin(Theta - m_ddTheta)*sin(Fi);
			z = m_cvPremik.GetZ() + m_dRadij*cos(Theta - m_ddTheta);
			cvTopVector.SetXYZ(x,y,z);
			cvTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);
			
			cpPloskev.SetBaseVector(cvBaseVector);
			cpPloskev.SetRightVector(cvRightVector);
			cpPloskev.SetTopRightVector(cvRightTopVector);
			cpPloskev.SetTopVector(cvTopVector);

			m_arPloskev.Add(cpPloskev);

			Fi += m_ddFi;
		}
		Theta -= m_ddTheta;
	}


	// zgornja vrsta trikotniki
	Fi = 0;
	Theta = m_ddTheta;
	for (i = 1; i < m_iSteviloPoldnevnikov; i++)
	{
		x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi);
		y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi);
		z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
		cvBaseVector.SetXYZ(x,y,z);
		cvBaseVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX() + m_dRadij*sin(Theta)*cos(Fi + m_ddFi);
		y = m_cvPremik.GetY() + m_dRadij*sin(Theta)*sin(Fi + m_ddFi);
		z = m_cvPremik.GetZ() + m_dRadij*cos(Theta);
		cvRightVector.SetXYZ(x,y,z);
		cvRightVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX();
		y = m_cvPremik.GetY();
		z = m_cvPremik.GetZ() + m_dRadij;
		cvRightTopVector.SetXYZ(x,y,z);
		cvRightTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		x = m_cvPremik.GetX();
		y = m_cvPremik.GetY();
		z = m_cvPremik.GetZ() + m_dRadij;
		cvTopVector.SetXYZ(x,y,z);
		cvTopVector.SetOddaljenostOdZaslona(m_dOddaljenostOdZaslona);

		cpPloskev.SetBaseVector(cvBaseVector);
		cpPloskev.SetRightVector(cvRightVector);
		cpPloskev.SetTopRightVector(cvRightTopVector);
		cpPloskev.SetTopVector(cvTopVector);

		m_arPloskev.Add(cpPloskev);

		Fi += m_ddFi;
	}
}

void CKrogla::TransformirajSeNaEkran()
{

}

// **************** IZRIS NA ZASLON

void CKrogla::NarisiSe(CPaintDC* pDC)
{
	RazdeliSe();    
					// dobil prog. za risanje
	int iLeft,iTop,iRight,iBottom;
	iLeft = 0;  //(int) 
	iTop = 0;
	iRight = 500;  //(int) 
	iBottom = 500;  //(int)
	CRect BaseRectangle (iLeft,
			             iTop,
					     iRight,
		    			 iBottom);

	pDC->Rectangle(&BaseRectangle);
	pDC->SetBkColor(RGB(0,0,255));     	
	// Create a new pen

	int iXSredinaPolja;//,iYSredinaPolja;
	iXSredinaPolja = (iRight + iLeft)/2;
//	iYSredinaPolja = (iTop + iBottom)/2;

	CPen MyNewPen;

	MyNewPen.CreatePen(PS_SOLID,
			           1,
					   RGB( 255, 0, 0));
    pDC->SelectObject(&MyNewPen);


	int iSize = m_arPloskev.GetSize();

	CPoint cpCurrentPoint,cpLastPoint;
	
	for (int i = (iSize - 1); i >= 0; i--)
	{
		cpCurrentPoint = m_arPloskev.GetAt(i).GetBaseVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		pDC->MoveTo(cpCurrentPoint);
		cpLastPoint = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetRightVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		pDC->LineTo(cpCurrentPoint);

		cpLastPoint = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetTopRightVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		pDC->LineTo(cpCurrentPoint);

		cpLastPoint = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetTopVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		pDC->LineTo(cpCurrentPoint);

		cpCurrentPoint = m_arPloskev.GetAt(i).GetBaseVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		pDC->LineTo(cpCurrentPoint);
		Sleep(m_uiDelay);
		cpLastPoint = cpCurrentPoint;
	}


}

// *************** BARVANJE POLIGONA

void CKrogla::NarisiSeSPoligoni(CPaintDC* pDC)
{
	RazdeliSe();
	SortX();

	int iLeft,iTop,iRight,iBottom;
	iLeft = 0;  //(int) 
	iTop = 0;
	iRight = 500;  //(int) 
	iBottom = 500;  //(int) 
	CRect BaseRectangle (iLeft,
			             iTop,
					     iRight,
		    			 iBottom);

	pDC->Rectangle(&BaseRectangle);
	pDC->SetBkColor(RGB(0,0,255));     	
	// Create a new pen

	int iXSredinaPolja;//,iYSredinaPolja;
	iXSredinaPolja = (iRight + iLeft)/2;
	//iYSredinaPolja = (iTop + iBottom)/2;

	CPen MyNewPen;

	MyNewPen.CreatePen(PS_SOLID,
			           1,
					   RGB( 255, 0, 0));
    pDC->SelectObject(&MyNewPen);


	int iSize = m_arPloskev.GetSize();

	CPoint cpCurrentPoint,cpLastPoint;
	CPoint *parPoint = new CPoint[4];
	for (int i = (iSize - 1); i >= 0; i--)
	{
		cpCurrentPoint = m_arPloskev.GetAt(i).GetBaseVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		parPoint[0] = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetRightVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		parPoint[1] = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetTopRightVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		parPoint[2] = cpCurrentPoint;
		cpCurrentPoint = m_arPloskev.GetAt(i).GetTopVector().GetTransformedPoint();
		cpCurrentPoint.x += iXSredinaPolja;
		//cpCurrentPoint.y += iYSredinaPolja;
		parPoint[3] = cpCurrentPoint;
		pDC->Polygon(parPoint,4);
		Sleep(m_uiDelay);
	}


	delete parPoint;
}

// ****************  IZPIS V DATOTEKO

void CKrogla::IzpisiSe()
{
	int iSize = m_arPloskev.GetSize();
	int i;
	FILE* fp;
	fp = fopen("Stirikotniki3D.txt","w");
	for (i = 0; i < iSize; i++)
	{
		fprintf(fp,"%f %f %f\n",m_arPloskev.GetAt(i).GetBaseVector().GetX(),m_arPloskev.GetAt(i).GetBaseVector().GetY(),m_arPloskev.GetAt(i).GetBaseVector().GetZ());	
		fprintf(fp,"%f %f %f\n",m_arPloskev.GetAt(i).GetRightVector().GetX(),m_arPloskev.GetAt(i).GetRightVector().GetY(),m_arPloskev.GetAt(i).GetRightVector().GetZ());	
		fprintf(fp,"%f %f %f\n",m_arPloskev.GetAt(i).GetTopRightVector().GetX(),m_arPloskev.GetAt(i).GetTopRightVector().GetY(),m_arPloskev.GetAt(i).GetTopRightVector().GetZ());	
		fprintf(fp,"%f %f %f\n",m_arPloskev.GetAt(i).GetTopVector().GetX(),m_arPloskev.GetAt(i).GetTopVector().GetY(),m_arPloskev.GetAt(i).GetTopVector().GetZ());	
		fprintf(fp,"\n\n");
	}

	fclose(fp);
	fp = fopen("Stirikkotnikik2D.txt","w");
	for (i = 0; i < iSize; i++)
	{
		fprintf(fp,"%i %i\n",m_arPloskev.GetAt(i).GetBaseVector().GetTransformedPoint().x,m_arPloskev.GetAt(i).GetBaseVector().GetTransformedPoint().y);	
		fprintf(fp,"%i %i\n",m_arPloskev.GetAt(i).GetRightVector().GetTransformedPoint().x,m_arPloskev.GetAt(i).GetRightVector().GetTransformedPoint().y);	
		fprintf(fp,"%i %i\n",m_arPloskev.GetAt(i).GetTopRightVector().GetTransformedPoint().x,m_arPloskev.GetAt(i).GetTopRightVector().GetTransformedPoint().y);	
		fprintf(fp,"%i %i\n",m_arPloskev.GetAt(i).GetTopVector().GetTransformedPoint().x,m_arPloskev.GetAt(i).GetTopVector().GetTransformedPoint().y);	
		fprintf(fp,"\n\n");
	}

	fclose(fp);
}

//   ***********  SORTIRANJE  ploskvic 

void CKrogla::SortX()   // po globini
{
	CPloskev Ploskev,TempPloskev;
	unsigned long i,ir=m_arPloskev.GetSize(),j,k,l=1,*istack;
	int jstack=0;
	double a;//,temp;
   
	istack = new unsigned long [NSTACK];

	for (;;) 
	{
		if (ir-l < M) 
		{
			for (j=l+1;j<=ir;j++) 
			{
				a=m_arPloskev[j-1].GetTezisceX();
				Ploskev = m_arPloskev[j-1];
				for (i=j-1;i>=l;i--) 
				{
					if (m_arPloskev[i-1].GetTezisceX() <= a) break;
					m_arPloskev[i+1-1]=m_arPloskev[i-1];
				}
				m_arPloskev[i+1-1]=Ploskev;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		}
		else 
		{
			k=(l+ir) >> 1;

			TempPloskev = m_arPloskev[k-1];
			m_arPloskev[k-1] = m_arPloskev[l+1-1];
			m_arPloskev[l+1-1] = TempPloskev;

			if (m_arPloskev[l-1].GetTezisceX() > m_arPloskev[ir-1].GetTezisceX()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l+1-1].GetTezisceX() > m_arPloskev[ir-1].GetTezisceX()) 
			{
				TempPloskev = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l-1].GetTezisceX() > m_arPloskev[l+1-1].GetTezisceX()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = TempPloskev;
			}
			i=l+1;
			j=ir;
			a=m_arPloskev[l+1-1].GetTezisceX();
			Ploskev = m_arPloskev[l+1-1];
			for (;;) 
			{
				do i++; while (m_arPloskev[i-1].GetTezisceX() < a);
				do j--; while (m_arPloskev[j-1].GetTezisceX() > a);
				if (j < i) break;

				TempPloskev = m_arPloskev[i-1];
				m_arPloskev[i-1] = m_arPloskev[j-1];
				m_arPloskev[j-1] = TempPloskev;
			}
			m_arPloskev[l+1-1]=m_arPloskev[j-1];
			m_arPloskev[j-1]=Ploskev;
			jstack += 2;
			if (jstack > NSTACK) MessageBox(NULL,"NSTACK too small in sort.", "TEAVE", MB_OK);
			if (ir-i+1 >= j-l) 
			{
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} 
			else 
			{
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	delete istack;
}

void CKrogla::SortZ()  // po viini
{
	CPloskev Ploskev,TempPloskev;
	unsigned long i,ir=m_arPloskev.GetSize(),j,k,l=1,*istack;
	int jstack=0;
	double a;//,temp;
   
	istack = new unsigned long [NSTACK];

	for (;;) 
	{
		if (ir-l < M) 
		{
			for (j=l+1;j<=ir;j++) 
			{
				a=m_arPloskev[j-1].GetTezisceZ();
				Ploskev = m_arPloskev[j-1];
				for (i=j-1;i>=l;i--) 
				{
					if (m_arPloskev[i-1].GetTezisceZ() <= a) break;
					m_arPloskev[i+1-1]=m_arPloskev[i-1];
				}
				m_arPloskev[i+1-1]=Ploskev;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		}
		else 
		{
			k=(l+ir) >> 1;
			TempPloskev = m_arPloskev[k-1];
			m_arPloskev[k-1] = m_arPloskev[l+1-1];
			m_arPloskev[l+1-1] = TempPloskev;

			if (m_arPloskev[l-1].GetTezisceZ() > m_arPloskev[ir-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l+1-1].GetTezisceZ() > m_arPloskev[ir-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l-1].GetTezisceZ() > m_arPloskev[l+1-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = TempPloskev;
			}
			i=l+1;
			j=ir;
			a=m_arPloskev[l+1-1].GetTezisceZ();
			Ploskev = m_arPloskev[l+1-1];
			for (;;) 
			{
				do i++; while (m_arPloskev[i-1].GetTezisceZ() < a);
				do j--; while (m_arPloskev[j-1].GetTezisceZ() > a);
				if (j < i) break;
				TempPloskev = m_arPloskev[i-1];
				m_arPloskev[i-1] = m_arPloskev[j-1];
				m_arPloskev[j-1] = TempPloskev;
			}
			m_arPloskev[l+1-1]=m_arPloskev[j-1];
			m_arPloskev[j-1]=Ploskev;
			jstack += 2;
			if (jstack > NSTACK) MessageBox(NULL,"NSTACK too small in sort.", "TEAVE", MB_OK);
			if (ir-i+1 >= j-l) 
			{
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} 
			else 
			{
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	delete istack;
}


void CKrogla::SortXBase()
{
	CPloskev Ploskev,TempPloskev;
	unsigned long i,ir=m_arPloskev.GetSize(),j,k,l=1,*istack;
	int jstack=0;
	double a;//,temp;
   
	istack = new unsigned long [NSTACK];

	for (;;) 
	{
		if (ir-l < M) 
		{
			for (j=l+1;j<=ir;j++) 
			{
				a=m_arPloskev[j-1].GetBaseVector().GetX();
				Ploskev = m_arPloskev[j-1];
				for (i=j-1;i>=l;i--) 
				{
					if (m_arPloskev[i-1].GetBaseVector().GetX() <= a) break;
					m_arPloskev[i+1-1]=m_arPloskev[i-1];
				}
				m_arPloskev[i+1-1]=Ploskev;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		}
		else 
		{
			k=(l+ir) >> 1;
			TempPloskev = m_arPloskev[k-1];
			m_arPloskev[k-1] = m_arPloskev[l+1-1];
			m_arPloskev[l+1-1] = TempPloskev;

			if (m_arPloskev[l-1].GetBaseVector().GetX() > m_arPloskev[ir-1].GetBaseVector().GetX()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l+1-1].GetBaseVector().GetX() > m_arPloskev[ir-1].GetBaseVector().GetX()) 
			{
				TempPloskev = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l-1].GetBaseVector().GetX() > m_arPloskev[l+1-1].GetBaseVector().GetX()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = TempPloskev;
			}
			i=l+1;
			j=ir;
			a=m_arPloskev[l+1-1].GetBaseVector().GetX();
			Ploskev = m_arPloskev[l+1-1];
			for (;;) 
			{
				do i++; while (m_arPloskev[i-1].GetBaseVector().GetX() < a);
				do j--; while (m_arPloskev[j-1].GetBaseVector().GetX() > a);
				if (j < i) break;
				TempPloskev = m_arPloskev[i-1];
				m_arPloskev[i-1] = m_arPloskev[j-1];
				m_arPloskev[j-1] = TempPloskev;
			}
			m_arPloskev[l+1-1]=m_arPloskev[j-1];
			m_arPloskev[j-1]=Ploskev;
			jstack += 2;
			if (jstack > NSTACK) MessageBox(NULL,"NSTACK too small in sort.", "TEAVE", MB_OK);
			if (ir-i+1 >= j-l) 
			{
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} 
			else 
			{
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	delete istack;
}

void CKrogla::SortZBase()
{
	CPloskev Ploskev,TempPloskev;
	unsigned long i,ir=m_arPloskev.GetSize(),j,k,l=1,*istack;
	int jstack=0;
	double a;//,temp;
   
	istack = new unsigned long [NSTACK];

	for (;;) 
	{
		if (ir-l < M) 
		{
			for (j=l+1;j<=ir;j++) 
			{
				a=m_arPloskev[j-1].GetTezisceZ();
				Ploskev = m_arPloskev[j-1];
				for (i=j-1;i>=l;i--) 
				{
					if (m_arPloskev[i-1].GetTezisceZ() <= a) break;
					m_arPloskev[i+1-1]=m_arPloskev[i-1];
				}
				m_arPloskev[i+1-1]=Ploskev;
			}
			if (jstack == 0) break;
			ir=istack[jstack--];
			l=istack[jstack--];
		}
		else 
		{
			k=(l+ir) >> 1;
			TempPloskev = m_arPloskev[k-1];
			m_arPloskev[k-1] = m_arPloskev[l+1-1];
			m_arPloskev[l+1-1] = TempPloskev;

			if (m_arPloskev[l-1].GetTezisceZ() > m_arPloskev[ir-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l+1-1].GetTezisceZ() > m_arPloskev[ir-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = m_arPloskev[ir-1];
				m_arPloskev[ir-1] = TempPloskev;
			}
			if (m_arPloskev[l-1].GetTezisceZ() > m_arPloskev[l+1-1].GetTezisceZ()) 
			{
				TempPloskev = m_arPloskev[l-1];
				m_arPloskev[l-1] = m_arPloskev[l+1-1];
				m_arPloskev[l+1-1] = TempPloskev;
			}
			i=l+1;
			j=ir;
			a=m_arPloskev[l+1-1].GetTezisceZ();
			Ploskev = m_arPloskev[l+1-1];
			for (;;) 
			{
				do i++; while (m_arPloskev[i-1].GetTezisceZ() < a);
				do j--; while (m_arPloskev[j-1].GetTezisceZ() > a);
				if (j < i) break;
				TempPloskev = m_arPloskev[i-1];
				m_arPloskev[i-1] = m_arPloskev[j-1];
				m_arPloskev[j-1] = TempPloskev;
			}
			m_arPloskev[l+1-1]=m_arPloskev[j-1];
			m_arPloskev[j-1]=Ploskev;
			jstack += 2;
			if (jstack > NSTACK) MessageBox(NULL,"NSTACK too small in sort.", "TEAVE", MB_OK);
			if (ir-i+1 >= j-l) 
			{
				istack[jstack]=ir;
				istack[jstack-1]=i;
				ir=j-1;
			} 
			else 
			{
				istack[jstack]=j-1;
				istack[jstack-1]=l;
				l=i;
			}
		}
	}
	delete istack;
}

